@@ -3,6 +3,18 @@ import os
 import requests
 import sys
 import time
+import datetime
+import copy
+
+FORMANT_REQUEST_ON_DEMAND_DATA_COMMAND_NAME = "formant.demand_data"
+
+
+def get_current_isodate():
+    return datetime.datetime.now(tz=datetime.timezone.utc).isoformat()
+
+
+def get_timestamp_str(dt: datetime.datetime):
+    return str(int(dt.timestamp()))
 
 
 class Client:
@@ -80,11 +92,24 @@ class Client:
         def call(token):
             headers = self._headers.copy()
             headers["Authorization"] = "Bearer %s" % token
-            response = requests.post(
-                "%s/queries" % self._query_api, headers=headers, json=params
-            )
-            response.raise_for_status()
-            return response.json()
+            # enable pagination by default
+            # TODO: support for aggregate queries
+            query = copy.deepcopy(params)
+            nextToken = None if len(query.get("aggregate", "")) > 0 else "true"
+            result = {"items": []}
+            while True:
+                if nextToken is not None:
+                    query["next"] = nextToken
+                response = requests.post(
+                    "%s/queries" % self._query_api, headers=headers, json=query
+                )
+                response.raise_for_status()
+                parsed = response.json()
+                nextToken = parsed.get("next")
+                result["items"] += parsed["items"]
+                if nextToken is None:
+                    break
+            return result
 
         return self._authenticate_request(call)
 
@@ -246,6 +271,26 @@ class Client:
 
         return self._authenticate_request(call)
 
+    def demand_device_data(
+        self,
+        device_id: str,
+        start: datetime.datetime,
+        end: datetime.datetime,
+    ):
+        params = {
+            "deviceId": device_id,
+            "command": FORMANT_REQUEST_ON_DEMAND_DATA_COMMAND_NAME,
+            "organizationId": self._organization_id,
+            "parameter": {
+                "meta": {
+                    "start": get_timestamp_str(start),
+                    "end": get_timestamp_str(end),
+                },
+                "scrubberTime": get_current_isodate(),
+            },
+        }
+        return self.create_command(params)
+
     def query_commands(self, params):
         """
         Query undelivered commands by device ID.
